module.exports = {
    root: true,
    parser: '@typescript-eslint/parser',
    globals: {
        _global: false,
        React: false, // 见 webpack.common.js 中的 webpack.ProvidePlugin
        PropTypes: false, // 同上
    },
    extends: [
        'plugin:@youzan/beautylint/react',
        'airbnb-base',
        'plugin:prettier/recommended',
        'plugin:react-hooks/recommended',
    ],
    rules: {
        // 1. 不允许使用 console.log，但是允许 console.warn 和 console.error
        'no-console': ['error', { allow: ['warn', 'error'] }],
        // 2. 允许引入 devDependencies 中的包，比如 webpack 相关的文件，这一个规则并不合理
        'import/no-extraneous-dependencies': ['error', { devDependencies: true }],
        // 3. 不限制 ++ 这种写法
        'no-plusplus': 'off',
        // 4. 允许使用断路（&& 或 ||）或三元运算的方式代替 if else
        'no-unused-expressions': ['error', { allowShortCircuit: true, allowTernary: true }],
        // 5. 不严格要求使用全等，因为现存代码中有大量的 ==，贸然改为 === 可能会导致 BUG。而且 == 有他的适用场景
        'eqeqeq': 'off',
        // 6. 没有必要限制使用解构还是直接取值
        'prefer-destructuring': 'off',
        // 7. 限制 _foo 这种标识符意义不大
        'no-underscore-dangle': 'off',
        // 8. 这个规则要求 import { someNamed } from 'someModule' 时 someModule 里有“export someNamed”的导出
        // 但是我们项目可以用这种方式引入 export default 出来的对象里面的某个属性，所以不做限制
        'import/named': 'off',
        // 9. airbnb-base 中推荐 class 中的普通方法都必须使用过 this，如果没有使用 this 就改为静态方法，这个我们暂不做要求
        'class-methods-use-this': 'off',
        // 10. 关闭 “不修改函数的参数”，这种情况在我们的项目中很多，暂时不做要求
        'no-param-reassign': 'off',
        // 11. 关闭 “函数要保证任何情况下都有返回值”，这种情况在我们的项目中很多，贸然加 return 可能会导致 BUG，暂时不做要求
        'consistent-return': 'off',
        // 12. 关闭 “switch 语句必须要有 default”，这种情况在我们的项目中很多，暂时不做要求
        'default-case': 'off',
        // 13. 关闭 “a 标签必须要有 href 属性”，有些人用 a 标签纯粹是为了让文字有主题色，这种情况在我们的项目中很多，暂时不做要求
        'jsx-a11y/anchor-is-valid': 'off',
        // 14. airbnb-base 中对 for in，for of，with 等语法做了严格限制，由于多处已存在这种写法，贸然改动会导致 BUG，这里我们暂不做要求
        'no-restricted-syntax': 'off',
        // 15. 同上
        'no-continue': 'off',
        // 16. Header_ 这种组件名不符合 React 对于组件需要使用 PascalCase 或 SCREAMING_SNAKE_CASE 方式命名的要求，但这里我们先不去限制这点
        'react/jsx-pascal-case': 'off',
        // 17. 关闭 "不允许嵌套三元运算"，现在代码中有一定数量的这种写法，贸然改动可能会导致BUG，且基本都是两个三元运算叠加在一起，并不是非常难以阅读，暂时不做要求
        'no-nested-ternary': 'off',
        // 18. 关闭 “一个文件中不允许出现的多个 class“，有现存代码这么写，不便于调整，暂时不改
        'max-classes-per-file': 'off',
        // 19. 现存这种代码较多，不便于调整，暂时不改
        'import/prefer-default-export': 'off',
        // 20. 同上
        'no-return-assign': 'off',
        // 21. 同上
        'no-shadow': 'off',
        // 22. 同上
        'no-case-declarations': 'off',
        // 23. 同上
        'import/no-cycle': 'off',
        // 24. airbnb-base 中不推荐使用 void，但是 void 0 代替 undefined 有他的好处
        'no-void': 'off',
        // 25. 意义不大，反而在解构多个对象属性时使得代码变得难看
        'prefer-const': 'off',
        // 26. 允许 jsx 中使用 globals 中指定的全局变量，比如 React.Fragment
        'react/jsx-no-undef': ['error', { allowGlobals: true }],
        // 27. 存量问题，贸然改动可能导致 BUG
        'guard-for-in': 'off',
        // 28. 同上，这是一个应该开启的规则，但无奈有很多处这种写法，如果改动会涉及到逻辑，风险很大
        'react/no-direct-mutation-state': 'off',
        // 29. 允许使用位运算
        'no-bitwise': 'off',
        // 30. 引入 JS 或 TS 等文件时不需要填写指定后缀，省略就可以了
        'import/extensions': [
            'error',
            'ignorePackages',
            {
                tsx: 'never',
                ts: 'never',
                jsx: 'never',
                js: 'never',
            },
        ],
        'react-hooks/exhaustive-deps': 'off',
        // 32. 这个要求有时候会导致逻辑看起来不直接
        'no-lonely-if': 'off',
        // 33. isNaN 和 Number.isNaN 是不一样的，这里会有坑
        'no-restricted-properties': 'off',
        // 34. JSX 中没有内容的自定义组件自动改为单标签，但原始的 html 标签不自动闭合
        'react/self-closing-comp': [
            'error',
            {
                component: true,
                html: false,
            },
        ],
        'no-script-url': 'off',
    },
    settings: {
        'import/extensions': ['.tsx', '.ts', '.jsx', '.js', '.json'], // 同 webpack 中的 resolve.extensions
        'import/resolver': {
            // 保证配置在别名下的模块被正确找到，避免 import/no-unresolved 的报错，已设置的别名参见 babel.config.js 中的 module-resolver 插件
            'babel-module': {},
            // 同上
            'typescript': {
                directory: '.', // tsconfig.json 所在目录
            },
        },
        // 原因同上，参见 babel.config.js 中的 babel-plugin-import 插件的设置，不过目前没找到适合 babel-plugin-import 的 resolver，有时间自己写一个，暂时先作为 core-modules
        'import/core-modules': ['@youzan/react-components', '@youzan/mei-components'],
    },
};
