const { promisify } = require("util"); // util 是一个Node.js 核心模块，提供常用函数的集合，用于弥补核心JavaScript 的功能 过于精简的不足
const figlet = promisify(require("figlet")); //  util.promisify() 这个方法,方便快捷的把原来的异步回调方法改成返回 Promise 实例的方法
const Metalsmith = require("metalsmith");
const inquirer = require("inquirer");
const path = require("path");
const clear = require("clear");
const chalk = require("chalk");
const log = (content) => console.log(chalk.green(content));
const ejs = require("ejs");
const fs = require("fs");
const open = require("open");
const { promise } = require("ora");
// const { clone } = require('./download')

const spawn = async (...args) => {
  const { spawn } = require("child_process");
  return new Promise((resolve) => {
    const proc = spawn(...args); // 在node.js中执行shell一般用spawn，实现从主进程的输出流连通到子进程的输出流
    proc.stdout.pipe(process.stdout); // 子进程正常流搭到主进程的正常流
    proc.stderr.pipe(process.stderr); // 子进程错误流插到主进程的错误流
    proc.on("close", () => {
      resolve();
    });
  });
};

function deleteFolderRecursive(url) {
  var files = [];
  /**
   * 判断给定的路径是否存在
   */
  if (fs.existsSync(url)) {
    /**
     * 返回文件和子目录的数组
     */
    files = fs.readdirSync(url);
    files.forEach(function (file, index) {
      var curPath = path.join(url, file);
      /**
       * fs.statSync同步读取文件夹文件，如果是文件夹，在重复触发函数
       */
      if (fs.statSync(curPath).isDirectory()) {
        // recurse
        deleteFolderRecursive(curPath);
      } else {
        fs.unlinkSync(curPath);
      }
    });
    /**
     * 清除文件夹
     */
    fs.rmdirSync(url);
  } else {
    console.log("给定的路径不存在，请给出正确的路径");
  }
}

module.exports = async (name, { ssh = "", destination = "" }) => {
  // 打印欢迎界面
  clear();
  const data = await figlet("Galio-cli"); // figlet是把文字变成大字
  log(data);

  // 获得命令运行时的路径
  const getCwd = () => process.cwd();

  // 生成项目文件
  const genFiles = async (options) => {
    // const templateSrc = path.resolve(__dirname, `../${options.name}`);
    const templateSrc = path.resolve(getCwd(), `./.${options.name}_backup`);

    options.NAME = options.name.toLocaleUpperCase();

    // //项目指定生成目录，如果命令中没有有配置目录，则在当前命令运行的目录下生成以项目名称为名字的新目录
    const destination = options.destination
      ? path.resolve(options.destination)
      : path.resolve(getCwd(), `${options.name}`);

    // 需要重新渲染的路径
    const renderPathList = [
      "vite.config.ts", // port
      "src/main.ts",
      ".env.microapp",
      ".env.web",
      ".env.web-prod",
      "index.html",
    ];

    console.log("=========templateSrc===========", templateSrc);

    return new Promise((resolve, reject) => {
      // https://metalsmith.io/
      Metalsmith(__dirname)
        .source(templateSrc)
        .destination(destination)
        // .destination(path.resolve(getCwd(), `./${options.name}`))
        .use((files) => {
          Object.keys(files).forEach((key) => {
            if (key.indexOf(".git") !== -1) {
              return;
            }
            if (renderPathList.includes(key)) {
              const file = files[key];
              // 原内容
              const str = file.contents.toString();
              console.log("files_____", key);
              // 新内容
              const newContents = ejs.render(str, options);
              // // 将新内容写到文件中
              file.contents = Buffer.from(newContents);
            }
          });
        })
        .build((err) => {
          if (err) {
            reject(err);
            console.error(err);
          } else {
            deleteFolderRecursive(templateSrc);

            resolve();
          }
        });
    });
  };

  const clone = async function (repo, name) {
    const ora = require("ora");
    const process = ora(`下载......${repo}`);
    process.start(); // 进度条开始
    await spawn("git", ["clone", repo, `.${name}_backup` || ""]);

    process.succeed();
  };

  const initHandler = async (answers) => {
    const { name, install_type, ssh } = answers;
    console.log({ ...answers, name: `${answers.name}`, destination });

    //根据回答以及选项，参数来生成项目文件
    const ora = require("ora");
    const processGenFiles = ora("Create project……");
    processGenFiles.start(); // 进度条开始

    const cwd = `./${name}`;

    await clone("https://gitlab.xiaoman.cc/perrysong/galio-template.git", name);

    await genFiles({ ...answers, destination });

    processGenFiles.succeed(
      chalk.green(`Create sub project complete: ${name}`)
    );
    log(`--------------------`);

    await spawn("git", ["init"], { cwd });

    await spawn("git", ["remote", "-v"], { cwd });

    // 安装 npm、yarn、pnpm
    const INSTALL_REGISTRY_URL = "--registry=http://npm.xiaoman.cn";
    if (install_type === "npm") {
      const processDownload = ora(`npm install......`);
      processDownload.start(); // 进度条开始
      await spawn("npm", ["install", INSTALL_REGISTRY_URL], {
        cwd,
      });
      processDownload.succeed();
      log("Installation dependency complete.");
    } else if (install_type === "yarn") {
      const processDownload = ora(`yarn install......`);
      processDownload.start(); // 进度条开始
      await spawn("yarn", ["install", INSTALL_REGISTRY_URL], {
        cwd,
      });
      processDownload.succeed();
      log("Installation dependency complete.");
    } else if (install_type === "pnpm") {
      const processDownload = ora(`pnpm install......`);
      processDownload.start(); // 进度条开始
      await spawn("pnpm", ["install", INSTALL_REGISTRY_URL], {
        cwd,
      });
      processDownload.succeed();
      log("Installation dependency complete.");
    }
    log(`================ INIT COMPLETE =================`);

    // // 自动打开浏览器
    // open(`http://localhost:8080`)

    const docUrl = `https://xmkm.yuque.com/docs/share/ccd718f2-3672-4936-aed0-a2d499e39e08`;
    // 自动启动项目'
    log(`子应用帮助文档：${docUrl}`);
    // open(docUrl)
    // // 自动启动项目
    // if (ssh) {
    //   await spawn("npm", ["run", "dev"], { cwd });
    // }
  };

  inquirer
    .prompt([
      {
        type: "input",
        name: "name",
        default: name,
        message: "Project name (<name>)",
        filter: function (val) {
          // 使用filter将回答变为小写
          return val.toLowerCase();
        },
      },
      {
        type: "input",
        name: "port",
        default: "8201",
        message: "Server port (default port (8201))",
        validate: function (val) {
          if (val.match(/\d/g)) {
            // 校验位数
            return true;
          }
          return "please enter a number";
        },
      },
      {
        type: "input",
        name: "description",
        default: "A Galio sub app",
        message: "Project description",
      },
      {
        type: "input",
        name: "author",
        message: "Author",
      },
      {
        type: "list",
        name: "install_type",
        message:
          "Should we run `npm install` for you after the project has been created?",
        choices: [
          {
            key: "pnpm",
            name: "Yes, use pnpm",
            value: "pnpm",
          },
          {
            key: "yarn",
            name: "Yes, use Yarn",
            value: "yarn",
          },
          {
            key: "npm",
            name: "Yes, use Npm",
            value: "npm",
          },
          {
            key: "no",
            name: "No, I will handle that myself",
            value: "no",
          },
        ],
      },
    ])
    .then((answers) => {
      initHandler({ ...answers });
    })
    .catch((error) => {
      console.error(error);
    });
};
